%define anolis_release 1
%define _hardened_build 1
# path to upstream git repository
%global git_upstream git://thekelleys.org.uk/dnsmasq.git
# tag of selected version
%global gittag v%{version}

# Attempt to prepare source-git with downstream repos
%bcond_with sourcegit
%bcond_without i18n

Name:           dnsmasq
Version:        2.89
Release:        %{anolis_release}%{?dist}
Summary:        A lightweight DHCP/caching DNS server

License:        GPL-2.0-only or GPL-3.0-only
URL:            http://www.thekelleys.org.uk/dnsmasq/
Source0:        %{url}%{name}-%{version}.tar.xz
Source1:        %{name}.service
Source2:        dnsmasq-systemd-sysusers.conf

Patch0001: 	dnsmasq-2.77-underflow.patch
Patch0002:	dnsmasq-2.81-configuration.patch
Patch0003:	dnsmasq-2.78-fips.patch
Patch0004: 	dnsmasq-CVE-2023-28450.patch
Patch0005: 	dnsmasq-2.90-dbus-watchers-bz2186468.patch

Requires:       nettle

Buildrequires:  gcc make gnupg2 systemd systemd-rpm-macros
BuildRequires:  dbus-devel libidn2-devel pkgconfig nettle-devel
BuildRequires:  pkgconfig(libnetfilter_conntrack)
%{?systemd_ordering}
%if %{with sourcegit}
BuildRequires:  git-core
%endif

%if %{with i18n}
BuildRequires: gettext
%endif

%description
Dnsmasq is lightweight, easy to configure DNS forwarder and DHCP server.
It is designed to provide DNS and, optionally, DHCP, to a small network.
It can serve the names of local machines which are not in the global
DNS. The DHCP server integrates with the DNS server and allows machines
with DHCP-allocated addresses to appear in the DNS with names configured
either in each host or in a central configuration file. Dnsmasq supports
static and dynamic DHCP leases and BOOTP for network booting of diskless
machines.

%if %{with i18n}
%package        langpack
Summary:        Translations for few languages
License:        LicenseRef-Fedora-Public-Domain AND GPL-2.0-or-later
BuildArch:      noarch
Requires:       %{name} = %{version}-%{release}
Supplements:    (%{name} = %{version}-%{release} and (langpacks-de or langpacks-es or langpacks-fi or langpacks-fr or langpacks-id or langpacks-it or langpacks-ka or langpacks-no or langpacks-pl or langpacks-pt_BR or langpacks-ro) )
 
%description    langpack
Translations for few languages on dnsmasq.
%endif

%package        utils
Summary:        Utilities for manipulating DHCP server leases
Requires:       %{name} = %{version}-%{release}

%description    utils
Utilities that use the standard DHCP protocol to query/remove a DHCP
server's leases.

%package doc
Summary:        Documentation files for %{name}
Requires:       %{name} = %{version}-%{release}
BuildArch:      noarch

%description    doc
The %{name}-doc package contains documentation files for %{name}.


%prep
%if %{with sourcegit}
%autosetup -n %{name}-%{version} -N -S git_am
# If preparing with sourcegit, drop again source directory
# and clone git repository
# FIXME: deleting just unpacked sources is dangerous
# But using %%setup changes used directories in %%build and %%install
rm -rf %{_builddir}/%{name}-%{version}
cd %{_builddir}
git clone -b %{gittag} %{git_upstream} %{name}-%{version}
cd %{name}-%{version}
git checkout -b rpmbuild
%else
%autosetup -n %{name}-%{version} -N
%endif
# Apply patches on top
%autopatch -p1

# use /var/lib/dnsmasq instead of /var/lib/misc
for file in dnsmasq.conf.example man/dnsmasq.8 man/es/dnsmasq.8 src/config.h; do
    sed -i 's|/var/lib/misc/dnsmasq.leases|/var/lib/dnsmasq/dnsmasq.leases|g' "$file"
done

#set default user /group in src/config.h
sed -i 's|#define CHUSER "nobody"|#define CHUSER "dnsmasq"|' src/config.h
sed -i 's|#define CHGRP "dip"|#define CHGRP "dnsmasq"|' src/config.h
sed -i "s|\(#\s*define RUNFILE\) \"/var/run/dnsmasq.pid\"|\1 \"%{_rundir}/dnsmasq.pid\"|" src/config.h

# optional parts
sed -i 's|^COPTS[[:space:]]*=|\0 -DHAVE_DBUS -DHAVE_LIBIDN2 -DHAVE_DNSSEC -DHAVE_CONNTRACK|' Makefile

%build
%make_build CFLAGS="$RPM_OPT_FLAGS" LDFLAGS="$RPM_LD_FLAGS" \
%if %{with i18n}
  all-i18n
%else
  all
%endif
%make_build -C contrib/lease-tools CFLAGS="$RPM_OPT_FLAGS" LDFLAGS="$RPM_LD_FLAGS"


%install
# normally i'd do 'make install'...it's a bit messy, though
mkdir -p $RPM_BUILD_ROOT%{_sbindir} \
        $RPM_BUILD_ROOT%{_mandir}/man8 \
        $RPM_BUILD_ROOT%{_var}/lib/dnsmasq \
        $RPM_BUILD_ROOT%{_sysconfdir}/dnsmasq.d \
        $RPM_BUILD_ROOT%{_sysconfdir}/dbus-1/system.d
install src/dnsmasq $RPM_BUILD_ROOT%{_sbindir}/dnsmasq
install dnsmasq.conf.example $RPM_BUILD_ROOT%{_sysconfdir}/dnsmasq.conf
install dbus/dnsmasq.conf $RPM_BUILD_ROOT%{_sysconfdir}/dbus-1/system.d/
install -m 644 man/dnsmasq.8 $RPM_BUILD_ROOT%{_mandir}/man8/
install -D trust-anchors.conf $RPM_BUILD_ROOT%{_datadir}/%{name}/trust-anchors.conf

# utils sub package
mkdir -p $RPM_BUILD_ROOT%{_bindir} \
         $RPM_BUILD_ROOT%{_mandir}/man1
install -m 755 contrib/lease-tools/dhcp_release $RPM_BUILD_ROOT%{_bindir}/dhcp_release
install -m 644 contrib/lease-tools/dhcp_release.1 $RPM_BUILD_ROOT%{_mandir}/man1/dhcp_release.1
install -m 755 contrib/lease-tools/dhcp_release6 $RPM_BUILD_ROOT%{_bindir}/dhcp_release6
install -m 644 contrib/lease-tools/dhcp_release6.1 $RPM_BUILD_ROOT%{_mandir}/man1/dhcp_release6.1
install -m 755 contrib/lease-tools/dhcp_lease_time $RPM_BUILD_ROOT%{_bindir}/dhcp_lease_time
install -m 644 contrib/lease-tools/dhcp_lease_time.1 $RPM_BUILD_ROOT%{_mandir}/man1/dhcp_lease_time.1

# Systemd
mkdir -p %{buildroot}%{_unitdir}
install -m644 %{SOURCE1} %{buildroot}%{_unitdir}
rm -rf %{buildroot}%{_initrddir}

#install systemd sysuser file
install -Dpm 644 %{SOURCE2} %{buildroot}%{_sysusersdir}/%{name}.conf

%if %{with i18n}
%make_install PREFIX=/usr install-i18n
%find_lang %{name} --with-man
%endif

%generate_compatibility_deps

%pre
#precreate users so that rpm can install files owned by that user
%sysusers_create_compat %{SOURCE2}

%post
%systemd_post dnsmasq.service

%preun
%systemd_preun dnsmasq.service

%postun
%systemd_postun_with_restart dnsmasq.service

%files
%dir %{abidir}
%doc setup.html dbus/DBus-interface
%license COPYING COPYING-v3
%defattr(0644,root,dnsmasq,0755)
%config(noreplace) %{_sysconfdir}/dnsmasq.conf
%dir %{_sysconfdir}/dnsmasq.d
%dir %attr(0755,root,dnsmasq) %{_var}/lib/dnsmasq
%defattr(-,root,root,-)
%config(noreplace) %{_sysconfdir}/dbus-1/system.d/dnsmasq.conf
%{_unitdir}/%{name}.service
%{_sbindir}/dnsmasq
%{abidir}/dnsmasq-option.list
%{_mandir}/man8/dnsmasq*
%dir %{_datadir}/%{name}
%{_datadir}/%{name}/trust-anchors.conf
%{_sysusersdir}/dnsmasq.conf

%files utils
%license COPYING COPYING-v3
%{_bindir}/dhcp_*
%{_mandir}/man1/dhcp_*
%{abidir}/dhcp_release6-option.list

%if %{with i18n}
%files langpack -f %{name}.lang
%endif

%files doc
%doc CHANGELOG FAQ doc.html

%changelog
* Mon Aug 14 2023 happy_orange <songnannan@linux.alibaba.com> - 2.89-1
- update to 2.89
- Limit offered EDNS0 size 1232 (CVE-2023-28450)

* Mon Jan 09 2023 Feng Su <sufeng@linux.alibaba.com> - 2.88-1
- update to v2.88

* Sun Oct 09 2022 mgb01105731 <mgb01105731@alibaba-inc.com> - 2.86-2
- add doc package

* Wed Mar 09 2022 Yuanhong Peng <yummypeng@linux.alibaba.com> - 2.86-1
- Init from upstream v2.86
